# Generated by Django 4.0.4 on 2022-05-06 06:47

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Label',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Scentific name')),
                ('taxonomic_rank', models.CharField(choices=[('order', 'Order'), ('family', 'Family'), ('genus', 'Genus'), ('species_group', 'Spec Group'), ('species', 'Species')], default='species', max_length=255, verbose_name='Taxonomic Rank')),
                ('phonetic_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Phonetic name')),
                ('phonetic_name_language', models.CharField(choices=[('de', 'German'), ('en', 'English'), ('undefined', 'Undefined')], default='undefined', max_length=255, verbose_name='Phonetic lanugage')),
                ('authors', models.CharField(blank=True, max_length=255, null=True, verbose_name='Authors')),
                ('kr_nr', models.CharField(blank=True, max_length=255, null=True, unique=True, validators=[django.core.validators.RegexValidator(message='K+R numbers should be alpha-numeric only!', regex=re.compile('[A-Za-z0-9]+'))], verbose_name='K+R-Nr.')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('last_update', models.DateTimeField(auto_now=True, verbose_name='Last modified at')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='labeltree.label', verbose_name='Parent')),
            ],
            options={
                'verbose_name': 'Label',
                'verbose_name_plural': 'Labels',
                'unique_together': {('name', 'taxonomic_rank')},
            },
        ),
    ]




from django.db import models


class BaseModel(models.Model):

    class Meta:
        abstract = True


    serializer_fields = (
        "id",
        "created", "last_update"
    )
    read_only_fields = (
        "created", "last_update"
    )

    ############################
    ##### Auto-Date Fields #####
    ############################

    created = models.DateTimeField(
        auto_now_add=True,
        verbose_name="Created at",
    )

    last_update = models.DateTimeField(
        auto_now=True,
        verbose_name="Last modified at",
    )




from __future__ import annotations

import requests
import typing as T

from dataclasses import dataclass
from dataclasses import fields
from munch import munchify
from urllib.parse import quote
from urllib.parse import urlencode

def _clean_name(name: str) -> str:
    return name.replace("-", " ").lower()

@dataclass
class GBIFEntry:
    key: int
    canonicalName: str
    rank: str

    parent: str
    parentKey: int

    synonym: bool
    issues: T.List

    vernacularNames: T.List

    vernacularLanguages = ["eng", "deu"]

    def __post_init__(self, *args, **kwargs):
        self.vernacularNames = [
            entry for entry in self.vernacularNames
            if entry["language"] in self.vernacularLanguages]

    def resolve(self):
        resp = GBIF.species(self.key)
        if "nubKey" in resp:
            newResp = GBIF.species(resp.nubKey)
            self.update(newResp)

    def update(self, entries):
        for field in fields(self):
            if field.name in entries:
                setattr(self, field.name, entries[field.name])

    @classmethod
    def new(cls, entries: dict) -> GBIFEntry:
        return cls(**{field.name: entries.get(field.name, None) for field in fields(cls)})

    @property
    def valid(self) -> bool:
        if self.synonym:
            return False
        resp = GBIF.species(self.key)
        backbone_issue = "BACKBONE_MATCH_NONE" in resp.issues

        # if has_issues:
        #     print(self)
        #     print(resp.issues)

        return not (backbone_issue or resp.synonym)

    @property
    def has_issues(self) -> bool:
        return self.issues is not None and len(self.issues) != 0


    def is_called(self, name):
        _ = _clean_name
        return any([_(entry.vernacularName) == _(name)
            for entry in self.vernacularNames])



class GBIF:
    URL = "https://api.gbif.org/v1/"

    @classmethod
    def get(cls, path: str, query: T.Optional = None) -> T.Dict:
        """ generic GET method """

        url = f"{cls.URL}{path}"

        if query is not None:
            query_str = urlencode(query, quote_via=quote)
            url = f"{url}?{query_str}"

        response = requests.get(url)
        response.raise_for_status()
        return response.json()

    @classmethod
    def match(cls, query: dict = None) -> dict:
        return munchify(cls.get("species/match", query))

    @classmethod
    def species(cls, speciesID: int, query: dict = None) -> dict:
        return munchify(cls.get(f"species/{speciesID}", query))

    @classmethod
    def search(cls, name: str, query: dict = None) -> dict:
        if query is None:
            query = {"q": name}
        else:
            query["q"] = name
        return munchify(cls.get(f"species/search", query))


    @classmethod
    def entries_by_name(self, name, query) -> GBIFEntry:

        resp = GBIF.search(name, query=query)

        entries = {}
        for result in resp["results"]:
            entry = GBIFEntry.new(result)

            if entry.canonicalName in entries:
                continue

            if not entry.is_called(name):
                continue

            entry.resolve()
            if not entry.valid:
                continue
            entries[entry.canonicalName] = entry


        if any([entry for entry in entries.values() if not entry.has_issues]):
            entries = {key: entry for key, entry in entries.items() if not entry.has_issues}

        return entries




from django.http import JsonResponse
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters
from rest_framework import routers
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response

from labeltree import models

class StandardResultsSetPagination(PageNumberPagination):
    page_size = 20
    page_size_query_param = 'page_size'
    max_page_size = 1000

class LabelViewSet(viewsets.ModelViewSet):
    serializer_class = models.LabelSerializer

    def get_queryset(self):
        return models.Label.objects.all()

    pagination_class = StandardResultsSetPagination

    filter_backends = [filters.SearchFilter, DjangoFilterBackend]
    filterset_fields = ['taxonomic_rank', "parent_id"]
    search_fields = ["name", "kr_nr", "parent__name", "group__name", "phonetic_name"]

class LabelGroupViewSet(viewsets.ModelViewSet):
    serializer_class = models.LabelGroupSerializer

    def get_queryset(self):
        return models.LabelGroup.objects.all()

    pagination_class = StandardResultsSetPagination

    filter_backends = [filters.SearchFilter, DjangoFilterBackend]
    filterset_fields = ["parent_id"]
    search_fields = ["name", "parent__name", "species__name"]

router = routers.DefaultRouter()
router.register(r"label", LabelViewSet, "label")
router.register(r"label-group", LabelGroupViewSet, "label-group")



# Generated by Django 4.0.4 on 2022-05-15 11:59

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('labeltree', '0002_alter_label_unique_together_alter_label_kr_nr_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='LabelGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('last_update', models.DateTimeField(auto_now=True, verbose_name='Last modified at')),
                ('name', models.CharField(max_length=255, verbose_name='Group name')),
                ('parent', models.ForeignKey(blank=True, limit_choices_to={'taxonomic_rank': 'genus'}, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='child_groups', to='labeltree.label', verbose_name='Parent')),
                ('species', models.ManyToManyField(limit_choices_to={'taxonomic_rank': 'species'}, related_name='group', to='labeltree.label')),
            ],
            options={
                'verbose_name': 'Label Group',
                'verbose_name_plural': 'Label Groups',
            },
        ),
    ]



from __future__ import annotations

import logging
import typing as T
import yaml

from itertools import starmap
from django.core.management.commands.runserver import Command as BaseCommand

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

from labeltree.models import Label
from labeltree.models import LabelGroup



class Command(BaseCommand):
    help = "Update label groups in the database based on the given YAML file."

    def add_arguments(self, parser):
        parser.add_argument('file_name')

    def handle(self, file_name, **kwargs):

        groups, species = SpeciesGroup.read(file_name)

        for group in groups:
            name = group.name
            species = Label.objects.filter(name__in=group.species)

            assert len(species) == len(group.species)

            parent = set([spec.parent for spec in species])

            assert len(parent) == 1

            parent = list(parent)[0]

            obj, created = LabelGroup.objects.get_or_create(name=name)

            obj.parent = parent
            obj.species.set(species)
            obj.save()


class SpeciesGroup(T.NamedTuple):
    name: str
    species: T.List[str]

    @classmethod
    def read(cls, file_name):
        group_list = []
        group_list_reverse = {}
        if file_name is not None:
            with open(file_name, "r") as f:
                group_list = cls.new(yaml.load(f, Loader))

            group_list_reverse = {species: group
                for group in group_list
                    for species in group.species
            }

        return group_list, group_list_reverse


    @staticmethod
    def check(name0, name1, sep=" / "):
        first0, *rest0 = name0.split(sep)
        first1, *rest1 = name1.split(sep)

        genus0, *first_spec0 = first0.split()
        genus1, *first_spec1 = first1.split()

        specs0 = set(first_spec0 + rest0)
        specs1 = set(first_spec1 + rest1)

        return genus0 == genus1 and (specs1.issubset(specs0) or specs0.issubset(specs1))


    @classmethod
    def new(cls, species_groups: T.Dict[str, T.List[str]]) -> T.List[SpeciesGroup]:
        return list(starmap(SpeciesGroup, species_groups.items()))
        # same as following:
        # return [
        #     SpeciesGroup(group_name, species) for group_name, species in species_groups.items()
        # ]


    def __eq__(self, group_or_name):

        if isinstance(group_or_name, SpeciesGroup):
            group_or_name = group_or_name.name

        if not isinstance(group_or_name, str):
            raise TypeError(f"Unsupported type: {type(group_or_name).__name__}")

        return SpeciesGroup.check(self.name, group_or_name)


    def __contains__(self, name):
        return name in self.species



from django.core.management.commands.runserver import Command as BaseCommand

from labeltree.management.commands.gbif_api import GBIF
from labeltree.management.commands.gbif_api import GBIFEntry

class Command(BaseCommand):
    help = "search for a specific"

    def add_arguments(self, parser):
        parser.add_argument("name")

    def handle(self, *args,
        name: str,
        **kwargs):

        entries = GBIF.entries_by_name(name, query=dict(rank="SPECIES"))
        print(entries)



from __future__ import annotations

import logging
import numpy as np
import requests
import typing as T


from collections import OrderedDict
from django.core.management.commands.runserver import Command as BaseCommand
from tqdm.auto import tqdm
from urllib.parse import quote
from urllib.parse import urlencode

from labeltree.management.commands.addgroups import SpeciesGroup
from labeltree.management.commands.gbif_api import GBIF
from labeltree.models import Label

def _nansafe(value):
    return None if value is np.nan else value


class Command(BaseCommand):
    help = "Update labels in the database based on the given CSV file."

    def add_arguments(self, parser):
        parser.add_argument('file_name')

        parser.add_argument('--sep', default=";")

        parser.add_argument('--taxon_ranks',
            nargs="*", default=("order", "family", "genus"))

        parser.add_argument("--order")
        parser.add_argument("--skip_by_name", action="store_true")

        parser.add_argument("--species_group_file",
            help="Path to a file containing species groupings")

    def handle(self, *args,
        file_name,
        sep: str,
        taxon_ranks: T.Tuple[str] = (Label.Rank.ORDER, Label.Rank.FAMILY, Label.Rank.GENUS),
        order: T.Optional[str] = None,
        skip_by_name: bool = False,
        species_group_file: str = None,
        **kwargs):


        content = np.loadtxt(file_name,
            dtype=[("name", "U255"), ("kr_nr", "U255")],
            delimiter=sep)

        group_list, _ = SpeciesGroup.read(species_group_file)

        entries = OrderedDict()
        db_entries = set(Label.objects.values_list("id", flat=True))
        db_names = set(Label.objects.values_list("name", flat=True))

        for i, entry in enumerate(tqdm(content)):
            name, kr_nr = entry["name"], entry["kr_nr"]

            if skip_by_name and name in db_names:
                continue

            if name in group_list:
                # species groups are handled in another command
                continue
                import pdb; pdb.set_trace()


            matched = GBIFEntry.match(name,
                kr_nr=kr_nr,
                order=order, ranks=taxon_ranks,
                already_matched=db_entries | entries.keys())

            for entry in matched:
                entries[entry.key] = entry
                try:
                    _new_label(entry)
                except Exception as e:
                    import pdb; pdb.set_trace()
                    raise



        labels = []
        for key, entry in entries.items():
            label, created = _new_label(entry, save=False)
            if created:
                labels.append(label)

        Label.objects.bulk_create(labels, batch_size=1000)

class GBIFEntry(T.NamedTuple):
    canonicalName: str
    rank: str
    authorship: str

    key: int
    parentKey: int

    kr_nr: T.Optional[str] = None


    @classmethod
    def new(cls, fields: dict) -> GBIFEntry:
        return cls(**{key: fields.get(key, None) for key in cls._fields})

    @classmethod
    def match(cls,
              name: str,
              kr_nr: T.Optional[str] = "",
              order: T.Optional[str] = None,
              ranks: T.Tuple[str] = (),
              already_matched: T.Set = set()) -> T.List[GBIFEntry]:

        result = []

        def _add_result(key, extra_fields={}):

            while True:
                if key in already_matched:
                    return

                fields = GBIF.get(f"species/{key}")

                if fields["rank"].lower() in Label.Rank:
                    break

                assert key != fields["parentKey"], \
                    f"Key was also set as parentKey: {key}"

                key = fields["parentKey"]

            entry = cls.new(fields | extra_fields)
            result.append(entry)

        query = dict(name=name)
        if order:
            query["order"] = order

        match = GBIF.get("species/match", query)

        assert match["matchType"] in ["EXACT", "FUZZY"], \
            f"Failed to match {name} ({match})!"

        synonymKey = None
        matchedKey = match["usageKey"]

        if match["synonym"]:
            synonymKey = matchedKey
            matchedKey = match["acceptedUsageKey"]

        matchedRank = match["rank"].lower()

        for rank in ranks:
            if rank == matchedRank:
                break

            _add_result(match[f"{rank}Key"])

        _add_result(matchedKey, dict(kr_nr=kr_nr))

        if synonymKey is not None:
            _add_result(synonymKey, dict(kr_nr=kr_nr))

        return result



def _new_label(entry: GBIFEntry, save: bool = True) -> Label:

    parent = entry.parentKey if entry.rank.lower() != Label.Rank.ORDER else None
    created = False
    try:
        label = Label.objects.get(
            id=entry.key,
            name=entry.canonicalName,
            taxonomic_rank=entry.rank.lower())


        assert label.parent_id == parent, \
            f"Data mismatch {label.parent=} != {parent=}"

        if label.kr_nr is not None:
            assert label.kr_nr == entry.kr_nr, \
                f"Data mismatch {label.kr_nr=} != {entry.kr_nr=} ({label=})"
        assert label.authors == entry.authorship, \
            f"Data mismatch {label.authors=} != {entry.authorship=} ({label=})"

    except Label.DoesNotExist:
        label = Label(
            id=entry.key,
            name=entry.canonicalName,
            taxonomic_rank=entry.rank.lower(),
            parent_id = parent,
            kr_nr = entry.kr_nr,
            authors = entry.authorship,
        )
        created = True


    if save:
        label.save()

    return label, created




from labeltree.models.label import Label
from labeltree.models.label import LabelSerializer
from labeltree.models.label_group import LabelGroup
from labeltree.models.label_group import LabelGroupSerializer


__all__ = [
	"Label",
	"LabelGroup",
	"LabelGroupSerializer",
	"LabelSerializer",
]



import logging
import numpy as np
import requests
import typing as T

from django.core.management.commands.runserver import Command as BaseCommand
from tqdm.auto import tqdm
from urllib.parse import quote
from urllib.parse import urlencode

from labeltree.management.commands.gbif_api import GBIF

class Command(BaseCommand):
    help = "Translates a list of vernacular names to a list of scientific names"
    URL = "https://api.gbif.org/v1/"


    def add_arguments(self, parser):
        parser.add_argument('file_name')
        parser.add_argument('--sep', default=";")
        parser.add_argument('--output')


    def handle(self, *args,
        file_name: str,
        sep: str,
        output: str,
        skip_by_name: bool = False,
        **kwargs):

        content = np.loadtxt(file_name,
            dtype=[("name", "U255"), ("kr_nr", "U255")],
            delimiter=sep,)

        result = []
        for i, entry in enumerate(tqdm(content)):
            name, kr_nr = entry["name"], entry["kr_nr"]

            entries = GBIF.entries_by_name(name, query={
                "rank": "SPECIES", "class": "Aves"
            })

            if len(entries) == 1:
                result.append(list(entries.keys())[0])

            elif len(entries) == 0:
                print(name)

            elif len(entries) != 1:
                print(name, entries)


        if output is None:
            print(result)
            return

        with open(output, "w") as f:
            for res, entry in zip(result, content):
                f.write(f"{res};{entry['kr_nr']}\n")



    @classmethod
    def _is_valid(cls, speciesID: int) -> bool:
        response = cls._get(f"species/{speciesID}")
        # either a synonym or has issues
        return response["synonym"] or len(response.get("issues", [])) != 0



import enum
import re

from django.core import validators
from django.db import models

from rest_framework import serializers

from labeltree.models import base

class Label(base.BaseModel):

    class Language(models.TextChoices):
        GERMAN = "de"
        ENGLISH = "en"
        UNDEFINED = "undefined"

    class Rank(models.TextChoices):
        ORDER = "order"
        FAMILY = "family"
        GENUS = "genus"
        SPECIES = "species"

    class Meta:
        verbose_name = "Label"
        verbose_name_plural = "Labels"

        unique_together = [
            ("name", "taxonomic_rank"),
            ("name", "kr_nr")
        ]

    def __str__(self):
        if self.kr_nr:
            return f"{self.name} {self.kr_nr} [{self.taxonomic_rank}]"

        return f"{self.name} [{self.taxonomic_rank}]"

    serializer_fields = base.BaseModel.serializer_fields + (
        "name", "taxonomic_rank", "parent_id",
        "kr_nr", "phonetic_name", "phonetic_name_language",
    )
    read_only_fields = base.BaseModel.read_only_fields + (
    )

    name = models.CharField(
        max_length=255,
        verbose_name="Scentific name",
    )

    taxonomic_rank = models.CharField(
        max_length=255,
        verbose_name="Taxonomic Rank",
        choices=Rank.choices,
        default=Rank.SPECIES,
    )

    parent = models.ForeignKey(
        to="self",
        verbose_name="Parent",
        related_name="children",
        related_query_name="child",
        on_delete=models.SET_NULL,
        blank=True,
        null=True
    )

    kr_nr = models.CharField(
        max_length=255,
        verbose_name="K+R-Nr.",
        validators=[validators.RegexValidator(
            regex=re.compile(r"[A-Za-z0-9]+"),
            message="K+R numbers should be alpha-numeric only!"
        )],
        blank=True,
        null=True,
    )

    phonetic_name = models.CharField(
        max_length=255,
        verbose_name="Phonetic name",
        blank=True,
        null=True
    )

    phonetic_name_language = models.CharField(
        max_length=255,
        verbose_name="Phonetic lanugage",
        choices=Language.choices,
        default=Language.UNDEFINED,
    )

    authors = models.CharField(
        max_length=255,
        verbose_name="Authors",
        blank=True,
        null=True
    )




class LabelSerializer(serializers.ModelSerializer):

    class Meta:
        model = Label
        fields = Label.serializer_fields
        read_only_fields = Label.read_only_fields



from django.db import models

from rest_framework import serializers

from labeltree.models import base
from labeltree.models import label

class LabelGroup(base.BaseModel):

    class Meta:
        verbose_name = "Label Group"
        verbose_name_plural = "Label Groups"

    def __str__(self):
        return super().__str__()

    serializer_fields = base.BaseModel.serializer_fields + (
        "name",
        "parent_id",
        "species",
    )

    read_only_fields = base.BaseModel.read_only_fields + (
        "species",
    )

    name = models.CharField(
        max_length=255,
        verbose_name="Group name",
    )

    parent = models.ForeignKey(label.Label,
        on_delete=models.SET_NULL,
        verbose_name="Parent",
        related_name="child_groups",
        blank=True, null=True,
        limit_choices_to=dict(taxonomic_rank=label.Label.Rank.GENUS),
    )

    species = models.ManyToManyField(label.Label,
        related_name="group",
        limit_choices_to=dict(taxonomic_rank=label.Label.Rank.SPECIES)
    )

class LabelGroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = LabelGroup
        fields = LabelGroup.serializer_fields
        read_only_fields = LabelGroup.read_only_fields



# Generated by Django 4.0.4 on 2022-05-11 12:01

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re


class Migration(migrations.Migration):

    dependencies = [
        ('labeltree', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='label',
            unique_together={('name', 'taxonomic_rank')},
        ),
        migrations.AlterField(
            model_name='label',
            name='kr_nr',
            field=models.CharField(blank=True, max_length=255, null=True, validators=[django.core.validators.RegexValidator(message='K+R numbers should be alpha-numeric only!', regex=re.compile('[A-Za-z0-9]+'))], verbose_name='K+R-Nr.'),
        ),
        migrations.AlterField(
            model_name='label',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', related_query_name='child', to='labeltree.label', verbose_name='Parent'),
        ),
        migrations.AlterUniqueTogether(
            name='label',
            unique_together={('name', 'kr_nr'), ('name', 'taxonomic_rank')},
        ),
    ]



from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

from labeltree.models import Label
from labeltree.models import LabelGroup



@admin.register(Label)
class LabelAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "parent_name",
        "children",
        "gbif_link",
        "kr_nr",
        "group_name",

        # "phonetic_name",
        # "phonetic_name_language",

        "authors",
    )


    list_filter = (
        'taxonomic_rank',
        ("group", admin.EmptyFieldListFilter),
        "created",
        "last_update",
    )

    ordering = ['kr_nr', "name"]
    search_fields = ['name', "kr_nr", "parent__name", "phonetic_name"]

    # def has_group(self, obj):
    #     return not obj.group.empty()

    @admin.display(empty_value='-')
    def group_name(self, obj):
        if not obj.group.exists():
            return
        group = obj.group.get()

        url = reverse('admin:labeltree_label_changelist')
        query = f"group__in={group.id}"
        return format_html(f"<a href='{url}?{query}'>{group.name}</a>")

    @admin.display(empty_value='-', ordering="id")
    def gbif_link(self, obj):
        url = f"https://api.gbif.org/v1/species/{obj.id}"
        name = f"{obj.taxonomic_rank.capitalize()} {obj.id}"
        return format_html(f"<a target='_new' href='{url}'>{name}</a>")

    @admin.display(empty_value='-', ordering="parent__name")
    def parent_name(self, obj):
        if obj.parent is None:
            return

        url = reverse('admin:labeltree_label_changelist')
        query = f"parent_id__exact={obj.parent_id}"
        name = obj.parent.name
        return format_html(f"<a href='{url}?{query}'>{name}</a>")

    @admin.display(empty_value='-')
    def children(self, obj):
        count = obj.children.count()
        if count == 0:
            return

        url = reverse('admin:labeltree_label_changelist')
        query = f"parent_id__exact={obj.id}"
        return format_html(f"<a href='{url}?{query}'>{count}</a>")


@admin.register(LabelGroup)
class LabelGroupAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "parent_name",
        "children"
    )
    ordering = ["name"]
    search_fields = ['name', "parent__name", "species__name"]

    list_filter = ("created", "last_update")


    @admin.display(empty_value='-', ordering="parent__name")
    def parent_name(self, obj):
        if obj.parent is None:
            return

        url = reverse('admin:labeltree_labelgroup_changelist')
        query = f"parent_id__exact={obj.parent_id}"
        name = obj.parent.name
        return format_html(f"<a href='{url}?{query}'>{name}</a>")

    @admin.display(empty_value='-')
    def children(self, obj):
        if obj.species.count() == 0:
            return

        url = reverse('admin:labeltree_label_changelist')
        query = f"group__in={obj.id}"
        name = ", ".join([spec.name for spec in obj.species.all()])
        return format_html(f"<a href='{url}?{query}'>{name}</a>")



from django.test import TestCase

# Create your tests here.

class ListLabels(TestCase):

    def test_something(self):
        self.assertEqual(2, 4)

    def test_something_else(self):
        self.assertEqual(4, 4)



# Generated by Django 4.0.4 on 2022-05-15 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labeltree', '0003_labelgroup'),
    ]

    operations = [
        migrations.AlterField(
            model_name='label',
            name='taxonomic_rank',
            field=models.CharField(choices=[('order', 'Order'), ('family', 'Family'), ('genus', 'Genus'), ('species', 'Species')], default='species', max_length=255, verbose_name='Taxonomic Rank'),
        ),
    ]
